@@ -27,11 +27,22 @@ import os
 
 from .ErrorMessage import error, debug
 from .Ports import get_port
-    
-def get_device(name, folder = ".", port = ""):
-    """returns a Device object as defined by the Device Class to be loaded"""
-    
-    # if no folder is given, the device class is loaded from the project folder
+
+
+def get_device(name, folder=".", port_string=""):
+    """
+    create a driver instance
+
+    Args:
+        name: Name of the driver being the name of the driver folder
+        folder: (optional) General folder to look for drivers, If folder is not used or empty, the driver is loaded
+            from the folder of the running script/project
+        port_string: (optional) A port resource name as selected in SweepMe! such as 'COM1', 'GPIB0::1::INSTR', etc.
+            It is required if the driver connects to an instrument and needs to open a specific port.
+
+    Returns:
+        Device object of the driver
+    """
 
     if folder == "":
         folder = "."
@@ -41,31 +52,32 @@ def get_device(name, folder = ".", port = ""):
         name = name[:-1]
 
     try:
-        Module = imp.load_source(name, folder + os.sep + name + os.sep + "main.py") # Loads .py file
+        # Loads .py file as module
+        module = imp.load_source(name, folder + os.sep + name + os.sep + "main.py")
     except:
-        raise Exception("Cannot load Device Class '%s' from folder %s. Please change folder or copy Device Class to your project." % (name, folder))
+        error()
+        raise Exception(f"Cannot load Driver '{name}' from folder {folder}. Please change folder or copy Driver "
+                        "to your project.")
  
-    device = Module.Device()
-    device._import_path = Module.__file__
+    device = module.Device()
+    device._import_path = module.__file__  # needs a comment
     
-    if port != "":
+    if port_string != "":
         if device.port_manager:
-            port = get_port(port, device.port_properties)
+            port = get_port(port_string, device.port_properties)
             device.port = port
 
-        device.set_parameters({"Port":port})
+        device.set_parameters({"Port": port_string})
         
     else:
         device.set_parameters()
 
     return device
-    
 
-def get_driver(name, folder = ".", port = ""):
+
+def get_driver(name, folder=".", port_string=""):
     """same function as get_device: returns a Device object as defined by the Device Class to be loaded"""
 
     # function is introduced because of a possible later renaming of DeviceClasses to Drivers
 
-    return self.get_device(name, folder, port)
-    
-            
\ No newline at end of file
+    return get_device(name, folder, port_string)
